/************************************************************************
 ***   GROOVESIZER DELTA v.112 - Granular Synth and 32-Step Sequencer
 ***   for the GROOVESIZER 8-Bit Musical Multiboard
 ***   http://groovesizer.com
 ***   Synth engine based on Auduino, the Lo-Fi granular synthesiser
 ***   by Peter Knight, Tinker.it http://tinker.it
 ***   Auduino help: http://code.google.com/p/tinkerit/wiki/Auduino
 ************************************************************************
 * Copyright (C) 2013 MoShang (Jean Marais) moshang@groovesizer.com
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/
 * 
 ************************************************************************/

#include <stdint.h>
#include <avr/interrupt.h>      // interrupts are required for the audio engine 
#include <avr/io.h>
#include <avr/pgmspace.h>       // included so we can use PROGMEM
#include <MIDI.h>               // MIDI library
#include <Wire.h>               // I2C library (required by the EEPROM library below)
#include <EEPROM24LC256_512.h>  // https://github.com/mikeneiderhauser/I2C-EEPROM-Arduino

EEPROM256_512 mem_1;            // define the eeprom chip
byte rwBuffer[64];              // our EEPROM read/write buffer

prog_uchar majorTriad[4]  PROGMEM = {
  0, 4, 7, 12}; // major triad
prog_uchar majorIntervals[4]  PROGMEM = {
  2, 5, 9, 11}; // other intervals of a major scale
prog_uchar minorTriad[4]  PROGMEM = {
  0, 3, 7, 12}; // minor triad
prog_uchar minorIntervals[4]  PROGMEM = {
  2, 5, 8, 11}; // other intervals of a minor scale
prog_uchar pentatonic[6]  PROGMEM = {
  0, 2, 4, 7, 9, 12}; // pentatonic

boolean noteOn = true;

// sequencer variables
byte seqTrueStep; // we need to keep track of the true sequence position (independent of sequence length)
volatile byte seqCurrentStep; // the current step - can be changed in other various functions and value updates immediately
char seqNextStep = 1; // change this to -1 to reverse pattern playback
byte seqLength = 32; // the length of the sequence from 1 step to 32 
byte seqNotes[32]; // the notes of the sequence - stores the current sequence's lookup values (1 - 127); a value of 0 means the step is not active
byte seqAutomate[192]; // stores parameter automation
boolean automate = false;
boolean updateAuto = false;
byte autoCounter = 0; // keeps count of the parameter automation
byte seqStepSelected[4]; // "1" means a step is selected, "0" means it's not 
// each byte corresponds to a row of button/leds/steps, each bit of the byte corresponds to a single step   
// it would have been easier to create an array with 32 bytes, but hey, we only need 4 with a bit of bitwise math
// and since we're already doing bitwise operations to shift in and out, we may as well 
byte seqStepMute[4]; // is the step muted or not; 4 bytes correspond to each of the rows of buttons - "0" is top, "3" is bottom
// show active steps and blink muted ones
// can only mute active (non-0 value)steps
byte seqStepTied[4]; // is the step tied to the next or not
byte seqTiedNote; // the current note lookup value of steps being tied
byte seqStepSlide[4]; // is the step marked as a slide
byte seqAccent[4]; // is the step accented
byte accentLevel = 0; // the amount by which to mark an accent
volatile boolean seqMidiStep; // advance to the next step with midi clock
byte bpm = 96;
unsigned long currentTime;
unsigned int clockPulse = 0; // keep count of the  pulses
byte clockPulseDur = (60000/bpm)/24; // the duration of one midi clock  pulse in milliseconds (6  pulses in a 16th, 12  pulses in an 8th, 24  pulses in a quarter)
unsigned long pulseStartTime = 0;
unsigned int sixteenthDur = (60000/bpm)/4; // the duration of a sixteenth note in milliseconds (6  pulses in a 16th, 12  pulses in an 8th, 24  pulses in a quarter)
unsigned int swing16thDur = 0; // a swung 16th's duration will differ from the straight one above
byte swing = 0; // the amount of swing from 0 - 255 
unsigned long sixteenthStartTime = 0;
boolean seqRunning = true; // is the sequencer running?
boolean stepGo = false; // should we fire the next step?
unsigned long offTime = 0; // when the note should be turned off in milliseconds
unsigned int onDur = 50; // how long the note should stay on

byte tempNote = 60; // a value that comes from pot6 an will be assigned as a note value for a step - default is middle-C

byte pulse; // to count incoming MIDI clocks

byte followAction = 0; // the behaviour of the pattern when it reaches the last step (0 = loop, 1 = next pattern, 2 = return to head, 3 = random chromatic, 4 = random major, 5 = random minor, 6 = random pentatonic)

byte head = 0; // the first in a series of chained patterns - any pattern triggered by hand is marked as the head (whether a followAction is set or not) 

boolean saved = false; // we need this variable for follow actions - only start doing the follow action once the pattern has been saved
// loading a pattern sets this variable to true - setting follow action to 1 or 2 sets this to false  

byte toc[16] = {
  0, 0, 0, 0,
  0, 0, 0, 0,  // 14 bytes to serve as table of contents (toc) for our 112 memory locations (stored as 14 bytes on eeprom page 448) 
  0, 0, 0, 0,  // add 2 bytes at the end for the sake of simplicity, but their values will never change from 0
  0, 0, 0, 0}; // these are the last two unused rows on the 4th triger page (trigPage = 3)      



int pageNum;            // which EEPROM page we'll read or save to  
boolean recall = false;         // a pattern change was requested with a buttonpress in pattern mode 

boolean clearMem = false; // have we marked a memory location to be erased?

// needed by savePatch()
boolean save = false;
unsigned long longPress = 0;

// how dense are the notes created in the pattern randomizer
byte noteDensity;

byte trigPage = 0; // which trigger page are we on?
byte controlLEDrow = B00000010; 

int pot1;
int pot2;
int pot3;
int pot4;
int pot5;
int pot6;

// the pots do more than one job and adjust a number of parameters, depending on the mode
// to avoid sudden jumps in values, we set the following variables to the current pot values when we change modes
// once a pot has moved through it's pickup value (the last stored value for the pot),
// we set the variables to "9999",  and can then start to accept values from the pot
// 9999 is an arbitrary value that is outside of the range of values the pot can have, and easy to check for
int decayPU = 9999;
int  grain2PhaseIncPU = 9999;
int grainPhaseIncPU = 9999;
int voice1PitchPU = 9999;
int voice2PitchPU = 9999;
int grainDecayPU = 9999;
int grain2DecayPU = 9999;
int bpmPU = 550; // pickup shouldn't be true (ie. 9999) at the start
int durationPU = 250;
int swingPU = 40;
int seqLengthPU = 40;

const byte pickup = 10; // how close does the current pot value have to be to the stored pickup value before pickup is true?  


// for the rows of LEDs
// the byte that will be shifted out 
byte LEDrow1;
byte LEDrow2;
byte LEDrow3;
byte LEDrow4;
byte LEDrow5;

// setup code for controlling the LEDs via 74HC595 serial to parallel shift registers
// based on ShiftOut http://arduino.cc/en/Tutorial/ShiftOut
const byte LEDlatchPin = 2;
const byte LEDclockPin = 6;
const byte LEDdataPin = 4;

// setup code for reading the buttons via CD4021B parallel to serial shift registers
// based on ShiftIn http://www.arduino.cc/en/Tutorial/ShiftIn
const byte BUTTONlatchPin = 7;
const byte BUTTONclockPin = 8;
const byte BUTTONdataPin = 9;

// needed by the button debouncer
// based on http://www.adafruit.com/blog/2009/10/20/example-code-for-multi-button-checker-with-debouncing/
#define DEBOUNCE 5
byte pressed[40], justpressed[40], justreleased[40], buttons[40];

// define the byte variables that the button states are read into
byte BUTTONvar1; 
byte BUTTONvar2; 
byte BUTTONvar3; 
byte BUTTONvar4;
byte BUTTONvar5;


// this determines what the buttons (and pots)are currently editing
byte buttonMode = 0;
// 0 = stepSelect
// 1 = stepMute
// 2 = stepTied
// 3 = stepSlide
// 4 = trigger mode
// 5 = pot-shift mode (alternate functions on the pots with right shift on), 
// 6 = pot-shift mode/accent page 
// 7 = preferences 

byte lastButtonMode = 0;

byte *edit[4]; // here come the pointers - things are getting a little tricky
// we want to be able to choose which of the byte-arrays in the sequencer struct we want to edit
// sadly, we can't simply assign a whole array to a variable ie. editThis = stepMute[];
// that means we'd have to duplicate a bunch of code
// we can, however, assign the address in memory of a byte in the arrays to a pointer variable eg. *edit
// (the star/asterisk indicates that we're declaring a pointer variable)
// like so: edit1 = &seqStepMute[1] (the & means we're assigning the address of seqStepMute[1], NOT the value is holds)
// when we use the variable again, we'll use an asterik again (dereferencing);
// good info on pointers here http://www.tutorialspoint.com/cprogramming/c_pointers.htm
// and here http://pw1.netcom.com/~tjensen/ptr/ch1x.htm

// 4051 for the potentiometers
// array of pins used to select 1 of 8 inputs on multiplexer
const byte select[] = {
  17,16,15}; // pins connected to the 4051 input select lines
const byte analogPin = 0;      // the analog pin connected to multiplexer output

// variables for changing the voices of each oscilator
byte voice1 = 0; 
byte lastVoice1 = 0;

int decay = 100; // like filter decay/release

//byte xorDistortion = 0; // the amount of XOR distortion - set by pot2 in pot-shift mode
byte distortion = 0; // either the XOR distortion amount, or a fixed value for accents
//int xorDistPU = 50; // pickup shouldn't be true at the start


byte nowPlaying = 255; // the pattern that's currently playing. 255 is a value we'll use to check for if no pattern is currently playing
byte cued = 255; // the pattern that will play after this one. 255 is a value we'll use to check for if no pattern is currently cued
byte confirm = 255; // the memory location that will be overwritten - needs to be confirmed. 255 means nothing to confirm
boolean cancelSave = false;



byte bpmTaps[5] = {
  60, 90, 120, 150, 180};

// some functions take place over multiple loops, we use these at the top of the loop to check if we're still busy   
boolean loadContinue = false;
//boolean saveContinue = false;
//boolean notifyContinue = false;

//preferences - these are saved on the same EEPROM page (448) as the toc
boolean midiNoteEntry; // send notes to the sequencer from incoming MIDI data (MIDI keyboard, etc.)
boolean midiNoteOut; // send out MIDI notes to the next device in the chain (you may want to switch this off, eg. if you have more than one groovesizer in a chain and only want to use notes from one)
boolean midiAutomationOut; // send out automation data as CC16
boolean midiSyncOut; // send out MIDI sync pulses
boolean midiTrigger; // send out trigger messages on channel 10 for slaved Groovesizers
boolean notify; // are we doing the scrolling text? 

byte triggerChannel = 10; // the MIDI channel that pattern change messages are sent and received on
byte automationTempChannel = 0;// used so we can set the channel to follow the value of noteChannel
byte automationChannel = 1; // the MIDI channel that automation is sent on
byte automationCC = 16; // the Control Change (CC) that automation is sent as 
byte noteChannel = 1; // the MIDI channel that pattern change messages are sent on

boolean showingNumbers = false; // are we busy displaying numbers (currently used for tempo nudge and on the preferences page)

// needed for midi sync
unsigned long lastMidiClock = 0;

boolean midiClock = false; // we are receiving MIDI clock - needed so we can hear notes when the sequencer isn't running, but we're getting clock from elsewhere

/**********************************************
 *****************AUDUINO CODE******************
 ***********************************************/
// incorporating DuaneB's volatile fix
uint16_t syncPhaseAcc;
volatile uint16_t syncPhaseInc;
uint16_t grainPhaseAcc;
volatile uint16_t grainPhaseInc;
uint16_t grainAmp;
volatile uint8_t grainDecay;
uint16_t grain2PhaseAcc;
volatile uint16_t grain2PhaseInc;
uint16_t grain2Amp;
volatile uint8_t grain2Decay;

#define PWM_PIN       3
#define PWM_VALUE     OCR2B
#define LED_PORT      PORTB
#define LED_BIT       5
#define PWM_INTERRUPT TIMER2_OVF_vect

// Smooth logarithmic mapping
prog_uint16_t antilogTable[] PROGMEM = {
  64830,64132,63441,62757,62081,61413,60751,60097,59449,58809,58176,57549,56929,56316,55709,55109,
  54515,53928,53347,52773,52204,51642,51085,50535,49991,49452,48920,48393,47871,47356,46846,46341,
  45842,45348,44859,44376,43898,43425,42958,42495,42037,41584,41136,40693,40255,39821,39392,38968,
  38548,38133,37722,37316,36914,36516,36123,35734,35349,34968,34591,34219,33850,33486,33125,32768
};
uint16_t mapPhaseInc(uint16_t input) {
  return (pgm_read_word_near(antilogTable + (input & 0x3f))) >> (input >> 6);
}
// the arrays below have been altered to go into PROGMEM (flash/program memory) instead of SRAM
// Stepped chromatic mapping
prog_uint16_t midiTable[] PROGMEM = {
  17,18,19,20,22,23,24,26,27,29,31,32,34,36,38,41,43,46,48,51,54,58,61,65,69,73,
  77,82,86,92,97,103,109,115,122,129,137,145,154,163,173,183,194,206,218,231,
  244,259,274,291,308,326,346,366,388,411,435,461,489,518,549,581,616,652,691,
  732,776,822,871,923,978,1036,1097,1163,1232,1305,1383,1465,1552,1644,1742,
  1845,1955,2071,2195,2325,2463,2610,2765,2930,3104,3288,3484,3691,3910,4143,
  4389,4650,4927,5220,5530,5859,6207,6577,6968,7382,7821,8286,8779,9301,9854,
  10440,11060,11718,12415,13153,13935,14764,15642,16572,17557,18601,19708,20879,
  22121,23436,24830,26306
};
uint16_t mapMidi(uint16_t input) {
  return (pgm_read_word_near(midiTable + (((1023-input) >> 4) + 30)));
}

void audioOn() {
  // Set up PWM to 31.25kHz, phase accurate
  TCCR2A = _BV(COM2B1) | _BV(WGM20);
  TCCR2B = _BV(CS20);
  TIMSK2 = _BV(TOIE2);
}

/**********************************************
 ***************END AUDUINO CODE****************
 ***********************************************/


// *************************************
//           THE SETUP
// *************************************

void setup() {
  /***********************************************
   *****************AUDUINO CODE******************
   ***********************************************/
  pinMode(PWM_PIN,OUTPUT);
  audioOn();
  /***********************************************
   ***************END AUDUINO CODE****************
   ***********************************************/

  //define pin modes
  pinMode(LEDlatchPin, OUTPUT);
  pinMode(LEDclockPin, OUTPUT); 
  pinMode(LEDdataPin, OUTPUT);

  pinMode(BUTTONlatchPin, OUTPUT);
  pinMode(BUTTONclockPin, OUTPUT); 
  pinMode(BUTTONdataPin, INPUT);

  for(byte bit = 0; bit < 3; bit++)
    pinMode(select[bit], OUTPUT);  // set the three EEPROM select pins to output

  clearAll(); // clear everything at the start, just to be on the safe side

  seqTrueStep = 0;
  seqCurrentStep = 0;

  // Initiate MIDI communications, listen to all channels
  MIDI.begin(MIDI_CHANNEL_OMNI);    

  // Connect the HandleNoteOn function to the library, so it is called upon reception of a NoteOn.
  MIDI.setHandleNoteOn(HandleNoteOn);  // only put the name of the function here - functions defined in HandleMidi

  MIDI.setHandleNoteOff(HandleNoteOff);

  MIDI.setHandleControlChange (HandleCC);

  MIDI.setHandleClock(HandleClock);

  MIDI.setHandleStart(HandleStart);

  MIDI.setHandleStop(HandleStop); 

  //begin I2C Bus
  Wire.begin();

  //begin EEPROM with I2C Address 
  mem_1.begin(0,0);//addr 0 (DEC) type 0 (defined as 24LC256)

  // check if character definitions exists in EEPROM by seeing if the first 10 bytes of the expected character array add up to 93
  mem_1.readPage(508, rwBuffer); // 0 is the first page of the EEPROM (511 is last) - a page is 64 bytes long
  byte x = 0;
  for (byte i = 0; i < 10; i++)
  {
    x = x + rwBuffer[i];
  }
  if (x != 93) // yikes, there's a problem - the EEPROM isn't set up properly
  {
    //give us a bad sign!
    LEDrow1 =  B10101010;
    LEDrow2 =  B01010101;
    LEDrow3 =  B10101010;
    LEDrow4 =  B01010101;
    LEDrow5 =  B10101010;
    for (byte i = 0; i < 100; i++)
    { 
      updateLeds();
      delay(5);
    }  
  }

  else
  {
    //give us a good sign!
    LEDrow1 =  B11111111;
    LEDrow2 =  B11111111;
    LEDrow3 =  B11111111;
    LEDrow4 =  B11111111;
    LEDrow5 =  B11111111;
    for (byte i = 0; i < 100; i++)
    { 
      updateLeds();
      delay(5);
    }
  }

  tocRead(); // update the toc array with a read from EEPROM page 448 - defined in HelperFunctions

  // message on boot
  if (notify)
    scrollText(" DELTA V112  "); // scrollText() is defined in HelperFunctions
}

// *************************************
//           THE AUDIO ENGINE
// *************************************

/***********************************************
 *****************AUDUINO CODE******************
 ***********************************************/
SIGNAL(PWM_INTERRUPT)
{

  uint8_t value;
  uint16_t output;

  syncPhaseAcc += syncPhaseInc;
  if (syncPhaseAcc < syncPhaseInc) {
    // Time to start the next grain
    grainPhaseAcc = 0;
    grainAmp = 0x7fff;
    grain2PhaseAcc = 0;
    grain2Amp = 0x7fff;
    LED_PORT ^= 1 << LED_BIT; // Faster than using digitalWrite
  }

  // Increment the phase of the grain oscillators
  grainPhaseAcc += grainPhaseInc;
  grain2PhaseAcc += grain2PhaseInc;

  // Convert phase into a triangle wave
  value = (grainPhaseAcc >> 7) & 0xff;
  if (grainPhaseAcc & 0x8000) value = ~value;
  // Multiply by current grain amplitude to get sample
  output = value * (grainAmp >> 8);

  // Repeat for second grain
  value = (grain2PhaseAcc >> 7) & 0xff;
  if (grain2PhaseAcc & 0x8000) value = ~value;
  output += value * (grain2Amp >> 8);

  // Make the grain amplitudes decay by a factor every sample (exponential decay)
  grainAmp -= (grainAmp >> 8) * grainDecay;
  grain2Amp -= (grain2Amp >> 8) * grain2Decay;

  // Scale output to the available range, clipping if necessary
  output >>= 9;
  if (output > 255) output = 255;

  // Output to PWM (this is faster than using analogWrite)  
  PWM_VALUE = output;
}
/***********************************************
 ***************END AUDUINO CODE****************
 ***********************************************/


// *************************************
//           THE LOOP
// *************************************

// welcome to the loop
void loop() {



  // the following variables are all marked static, because we only want to create them once the first time through the loop
  // we create the variables here and not before the setup, because they only apply to the loop (they don't have global scope)

  static int pressedPot1;

  static int longPressDur = 300; // the expected duration of a long press in milliseconds


  // blink the led for each step in the sequence
  static byte stepLEDrow1;
  static byte stepLEDrow2;
  static byte stepLEDrow3;
  static byte stepLEDrow4;

  // controlLEDrow is defined as global variable above so we can set it from helper functions

  static unsigned long ledBlink; // for blinking an LED

  static unsigned long ledBlink2; // for blinking another LED

  static byte selector;

  static boolean shiftL;
  static boolean shiftR;

  static boolean on = true; // a little helper because my logic sucks
  static boolean slide = false; // and another one

    static int pitchTarget = 0; // the pitch we're aiming for when we slide

  static byte midiOn; // the note we're sending out via MIDI 

  // Call MIDI.read the fastest you can for real-time performance.
  MIDI.read();
  // there is no need to check if there are messages incoming if they are bound to a Callback function. 



  // we check buttons at the top of the loop, the "just" flags get reset here every time
  // it's best to reset the "just" flags manually whenever we check for "just" and find it true - use "clearJust()"
  // it prevents another just function from being true further down in the same loop
  check_switches(); //defined in ButtonCheck in the tab above

  // get the pot values
  getPotValues(); // defined in HelperFunctions 

    // reset the LEDs to all off at the start of the loop - defined in HelperFunctions (see tab above)
  ledsOff();

  if (!noteOn) // like filter decay release
  {
    grainPhaseInc = (grainPhaseInc > decay) ? grainPhaseInc - decay : 0;
    grain2PhaseInc = (grain2PhaseInc > decay) ? grain2PhaseInc - decay : 0;
  }

  // *************************************
  //     PREFERENCES PAGE inside the loop
  // *************************************

  if (buttonMode == 7) // we're on the preferences page
  {
    static int potLock5 = 0;
    static int potLock6 = 0; 

    LEDrow5 =  B10000001; // show we're on the preferences page

    if (justpressed[33])
    {
      midiSyncOut = !midiSyncOut;
      if (midiSyncOut && midiAutomationOut)
        midiAutomationOut = false;
      clearJust();
      if (notify)
        scrollText(" SYNC OUT  ");
    }
    else if (justpressed[34])
    {
      midiAutomationOut = !midiAutomationOut;
      if (midiSyncOut && midiAutomationOut)
        midiSyncOut = false;
      clearJust();
      if (notify)
        scrollText(" AUTOMATION OUT  ");
      potLock5 = pot5;
      potLock6 = pot6;
    }
    else if (justpressed[35])
    {
      midiNoteOut = !midiNoteOut;
      clearJust();
      if (notify)
        scrollText(" NOTE OUT  ");
      potLock6 = pot6;
    }
    else if (justpressed[36])
    {
      midiNoteEntry = !midiNoteEntry;
      clearJust();
      if (notify)
        scrollText(" NOTE ENTRY  ");
    }
    else if (justpressed[37])
    {
      midiTrigger = !midiTrigger;
      clearJust();
      if (notify)
        scrollText(" TRIGGER  ");
      potLock6 = pot6;
    }
    else if (justpressed[38])
    {
      notify = !notify;
      clearJust();
      if (notify)
        scrollText(" NOTIFY  ");
    }

    else if (justpressed[32] || justpressed[39]) // exit the preferences page
    {
      if (automationTempChannel == 0)
        automationChannel = noteChannel;
      savePrefs(); // write the preferences and rewrite the toc (they're on the same EEPROM page - 448)
      buttonMode = 0; //lastButtonMode;
      clearJust();
      shiftR = true;
      shiftL = true;
      seqRunning = true;
    }

    else if (pressed[34])
    {
      if (difference(pot6, potLock6) > 20 && difference(pot5, potLock5) <= 20)
      {
        potLock6 = 2000; // make sure the difference is safely more than 20
        //potLock5 = pot5;
        automationTempChannel = map((constrain(pot6, 40, 1000)), 40, 1000, 0, 16);
        automationChannel = automationTempChannel;
        showNumber(automationChannel);
        showingNumbers = true;
        midiAutomationOut = true;
        midiSyncOut = false;
      }      
      else if (difference(pot5, potLock5) > 20)
      {
        potLock5 = 2000; // make sure the difference is safely more than 20
        //potLock6 = pot6;
        automationCC = map((constrain(pot5, 40, 1000)), 40, 1000, 0, 127);
        showNumber(automationCC);
        showingNumbers = true;
        midiAutomationOut = true;
        midiSyncOut = false;
      }
    }

    else if (justreleased[34])
    {
      showingNumbers = false;
      clearJust();
    }

    else if (pressed[35])
    {
      if (difference(pot6, potLock6) > 20)
      {
        potLock6 = 2000; // make sure the difference is safely more than 20
        noteChannel = map((constrain(pot6, 40, 1000)), 40, 1000, 1, 16);
        showNumber(noteChannel);
        showingNumbers = true;
        midiNoteOut = true;
      }
    }

    else if (justreleased[35])
    {
      showingNumbers = false;
      clearJust();
    }

    else if (pressed[37])
    {
      if (difference(pot6, potLock6) > 20)
      {
        potLock6 = 2000; // make sure the difference is safely more than 20
        triggerChannel = map((constrain(pot6, 40, 1000)), 40, 1000, 1, 16);
        showNumber(triggerChannel);
        showingNumbers = true;
        midiTrigger = true;
      }
    }

    else if (justreleased[37])
    {
      showingNumbers = false;
      clearJust();
    }

    if (!showingNumbers)
    {
      // update the control row with the selected preferences before updating the leds
      if(midiSyncOut)
        LEDrow5 = bitSet(LEDrow5, 1);
      if(midiAutomationOut)
        LEDrow5 = bitSet(LEDrow5, 2);
      if(midiNoteOut)
        LEDrow5 = bitSet(LEDrow5, 3);      
      if (midiNoteEntry)
        LEDrow5 = bitSet(LEDrow5, 4);
      if (midiTrigger)
        LEDrow5 = bitSet(LEDrow5, 5);
      if(notify)
        LEDrow5 = bitSet(LEDrow5, 6);
    }

    updateLeds();
  }

  // *************************************
  //     end of the PREFERENCES PAGE
  // *************************************

  else // we're not on the preferences page
  {
    // are we busy loading a pattern?
    if (loadContinue)
    {
      loadPatch(cued);
    }

    if (buttonMode != lastButtonMode) // the buttonmode has changed since we were last here, so let's update the pickup values
    {
      clearSelected(); // we don't want to carry over selected steps between button modes
      if (lastButtonMode < 5) // ie. not mode 5 or 6 
      {
        // update the pickup values only if pickup is actually true for a variable (ie. the pot has already moved through its pickup value)
        // this is a nice place to use the ternary operator (a compact if-else statement) http://www.cplusplus.com/forum/articles/14631/


        grainPhaseIncPU = (grainPhaseIncPU == 9999) ? pot1 : grainPhaseIncPU;
        grainDecayPU = (grainDecayPU == 9999) ? pot2 : grainDecayPU;
        grain2PhaseIncPU = (grain2PhaseIncPU == 9999) ? pot3 :  grain2PhaseIncPU;
        grain2DecayPU = (grain2DecayPU == 9999) ? pot4 : grain2DecayPU;
        decayPU = (decayPU == 9999) ? pot5 : decayPU; 
        voice1PitchPU = (voice1PitchPU == 9999) ? constrain(pot6, 40, 1000) : voice1PitchPU;
        voice2PitchPU = (voice2PitchPU == 9999) ? constrain(pot4, 40, 1000) : voice2PitchPU;
      }
      else if (lastButtonMode == 5)// we were in pot-shift mode last
      { 
        // assign new pickup values if pickups are true
        bpmPU = (bpmPU == 9999) ? constrain(pot6, 40, 1000) : bpmPU;
        durationPU = (durationPU == 9999) ? constrain(pot5, 40, 1000) : durationPU;
        swingPU = (swingPU == 9999) ? constrain(pot4, 40, 1000) : swingPU;
        seqLengthPU = (seqLengthPU == 9999) ? constrain(pot3, 40, 1000) : seqLengthPU;
        //xorDistPU = (xorDistPU == 9999) ? pot2 : xorDistPU;
      }
    }

    if (buttonMode < 5) // ie. not mode 5 or 6
    {
      tempNote = map(constrain(pot6, 40, 1000), 40, 1000, 36, 92); // shave it down to 4 octaves for better accuracy
      //tempNote = map(tempNote, 0, 127, 0, 54); // for pentatonic note table

      if (decayPU == 9999)   
        decay = 1023 - pot5;
      else if (difference(pot5, decayPU) < pickup)
        decayPU = 9999;


      if ( grain2PhaseIncPU == 9999)
      {
        if (noteOn)
        {
          if (checkAccent(seqCurrentStep)) // does the step have an accent?
          {
            if (seqRunning || midiClock)
              grain2PhaseInc = mapPhaseInc(1023 - (768 + accentLevel)) / 2;
          }
          else
          {
            if (seqRunning || midiClock)   
              grain2PhaseInc = mapPhaseInc(1023 - pot3) / 2;
          }
        }
      }
      else if (difference(pot3,  grain2PhaseIncPU) < pickup)
        grain2PhaseIncPU = 9999;
      else // pickup is not yet true, but we still have to play something!
      {
        if (noteOn)
        {
          if (checkAccent(seqCurrentStep)) // does the step have an accent?
          {
            if (seqRunning || midiClock)
              grain2PhaseInc = mapPhaseInc(1023 - (768 + accentLevel)) / 2;
          }
          else
          {
            if (seqRunning || midiClock) 
              grain2PhaseInc = mapPhaseInc(1023 - grain2PhaseIncPU) / 2;
          }
        }
      }

      if (grainPhaseIncPU == 9999)
      {
        if (noteOn && !automate  && (seqRunning || midiClock))
        {
          grainPhaseInc  = mapPhaseInc(512 - (pot1 >> 1)) / 2;
        }  
      }
      else if (difference(pot1, grainPhaseIncPU) < pickup)
        grainPhaseIncPU = 9999;
      else // pickup is not yet true, but we still have to play something!
      {
        if (noteOn && !automate  && (seqRunning || midiClock))
        {
          grainPhaseInc  = mapPhaseInc(512 - (grainPhaseIncPU >> 1)) / 2;
        }
      }
    }

    if (justpressed[39])
    {
      pressedPot1 = pot1;
      clearJust(); 
    }

    if (buttonMode < 5) // if we're not in pot-shift mode
    {
      if (!pressed[39])
      {
        if (automate) // automation is on
        {
          if (noteOn)
            grainPhaseInc  = mapPhaseInc(512 - ((int)seqAutomate[autoCounter] << 2))/ 2;
          // osc2.incr = map(seqAutomate[autoCounter], 0, 127, osc1.incr, 4*osc1.incr);
          if(midiAutomationOut)
            MIDI.sendControlChange(automationCC, seqAutomate[autoCounter], automationChannel);
        } 
      }
      else // right-shift is pressed, so assign the values to the automation array
      {
        if (!automate)
        {
          if (difference(pressedPot1, pot1) > 100)
          {
            automate = true;
            updateAuto = true;  
          }
          else
          {
            if (seqRunning || midiClock)
              grainPhaseInc = mapPhaseInc(512 - (pot1 >> 1)) / 2;
            // osc2.incr = map(constrain(pot4, 40, 1000), 40, 1000, osc1.incr, 4*osc1.incr); 
          }
        }
        else if (!updateAuto)
        {
          if (difference(pressedPot1, pot1) > 100)
            updateAuto = true;
          else
          {
            if (noteOn && (seqRunning || midiClock))
              grainPhaseInc  = mapPhaseInc(512 - ((int)seqAutomate[autoCounter] << 2))/ 2;
            //  osc2.incr = map(seqAutomate[autoCounter], 0, 127, osc1.incr, 4*osc1.incr);
            if(midiAutomationOut)
              MIDI.sendControlChange(automationCC, seqAutomate[autoCounter], automationChannel);
          }    
        }
        else // so both automate and updateAuto are true
        {
          shiftR = true;
          if (noteOn && (seqRunning || midiClock))
            grainPhaseInc  = mapPhaseInc(512 - (pot1 >> 1)) / 2;
          // osc2.incr = map(constrain(pot4, 40, 1000), 40, 1000, osc1.incr, 4*osc1.incr);
          seqAutomate[autoCounter] = map(pot1, 0, 1023, 0, 127);
          MIDI.sendControlChange(automationCC, seqAutomate[autoCounter], automationChannel);    
        }
      }

      if (grainDecayPU == 9999)
      {
        grainDecay = (1023 - pot2) / 8;
      }
      else if (difference(grainDecayPU, pot2) < pickup)
        grainDecayPU = 9999; 
      else // pickup is not yet true, but we still have to play something!
      {
        grainDecay = (1023 - grainDecayPU) / 8;
      }

      if (grain2DecayPU == 9999)
      {
        grain2Decay = (1023 - pot4) / 4;
      }
      else if (difference(grain2DecayPU, pot4) < pickup)
        grain2DecayPU = 9999; 
      else // pickup is not yet true, but we still have to play something!
      {
        grain2Decay = (1023 - grain2DecayPU) / 4;
      }
    }

    else // ie. we are in pot-shift mode
    {
      if (!automate) // automation is off
      {
        if (noteOn)
          grainDecay = (1023 - grainDecayPU) / 8;
        //  osc2.incr = map(constrain(voice2PitchPU, 40, 1000), 40, 1000, osc1.incr, 4*osc1.incr); 
      }
      else // automation is on
      {
        if (noteOn && (seqRunning || midiClock))
          grainPhaseInc  = mapPhaseInc(512 - ((int)seqAutomate[autoCounter] << 2))/ 2;
        // osc2.incr = map(seqAutomate[autoCounter], 0, 127, osc1.incr, 4*osc1.incr);
        if(midiAutomationOut)
          MIDI.sendControlChange(automationCC, seqAutomate[autoCounter], automationChannel);
      }

      grain2Decay = (1023 - grain2DecayPU) / 4;
    }

    if (buttonMode > 4) // we're in pot-shift mode
    {   
      if (seqRunning) // only allow changes to the tempo if the sequencer is running (ie. we're not slaved to mid clock)
      {
        if (bpmPU == 9999) // change the bpm with pot6
        {
          bpm = constrain(pot6>>2, 45, 220);
          bpmChange();
        }
        else if (difference(bpmPU, pot6) < pickup)
          bpmPU = 9999;
        if (durationPU == 9999) // change the duration of a note with pot 5
          onDur = map(constrain(pot5, 40, 1000), 40, 1000, 15, (sixteenthDur - 10));
        else if (difference(durationPU, pot5) < pickup)
          durationPU = 9999;
      }

      if (swingPU == 9999) // change the amount of swing with pot 4
      {
        swing = map(constrain(pot4, 40, 1000), 40, 1000, 0, 255);
      }
      else if (difference(swingPU, pot4) < pickup)
        swingPU = 9999;

      if (seqLengthPU == 9999) // change the length of the pattern
        seqLength = map(constrain(pot3, 40, 1000), 40, 1000, 1, 32);
      else if (difference(seqLengthPU, pot3) < pickup)
        seqLengthPU = 9999;

      if (buttonMode == 5)
      {
        /*
        if (xorDistPU == 9999)
         xorDistortion = constrain((pot2>>2) - 50, 0, 256); //make it an 8 bit value instead of 10 bit value
         else if (difference(xorDistPU, pot2) < pickup)
         xorDistPU = 9999;
         */
      }
      else // we're in buttonMode6
      {
        // take the accent level from pot 2
        accentLevel = map(pot2, 0, 1023, 0, 255);
      }
      noteDensity = map(constrain(pot1, 40, 1000), 40, 1000, 20, 100);
    }

    lastButtonMode = buttonMode; // keep track of what the last buttonmode was so we know when it changes
    // below we check to see if we're in one of the buttonmodes that requires us to change which array the edit bytes are affecting

    switch (buttonMode)
    {
    case 0: // selecting steps
      for (byte i = 0; i < 4; i++)
        edit[i] = &seqStepSelected[i];
      controlLEDrow = B00000010;
      break;
    case 1: // selecting which steps are active
      for (byte i = 0; i < 4; i++)
        edit[i] = &seqStepMute[i];
      controlLEDrow = B00000100;   
      break;
    case 2: // selecting which steps are tied
      for (byte i = 0; i < 4; i++)
        edit[i] = &seqStepTied[i];
      controlLEDrow = B00001000;
      break;
    case 3: // selecting which steps have slides
      for (byte i = 0; i < 4; i++)
        edit[i] = &seqStepSlide[i];
      controlLEDrow = B00010000;
      break;
    case 5: // selecting steps in pot-shift mode
      for (byte i = 0; i < 4; i++)
        edit[i] = &seqStepSelected[i];
      controlLEDrow = B10000000;
      break;
    case 6: // selecting which steps have accents
      for (byte i = 0; i < 4; i++)
        edit[i] = &seqAccent[i];
      controlLEDrow = B10100000;
      break;
    }

    // *************************************
    //     BUTTON CHECKS inside the LOOP
    //     start of the button check if/else structure
    //     in the if/else structure, things are mutually exclusive
    //     in other words, only one of the following things can be true for each run through the loop
    // *************************************
    if (!(justreleased[32] && shiftL) && !(justreleased[39] && shiftR)) // we're using this to bracket the entire button-check if/else structure
      // if we've just released 32 with shiftL on, deal with it first
      // if we've just released 39 with shiftR on, deal with it first
    {
      //******COMMON******
      if (justreleased[32])
      {
        if (buttonMode != 4)
        {
          buttonMode = 4; // trigger mode
          clearSelected();
          controlLEDrow = B00000001; // light the LED for trigger mode
          controlLEDrow = bitSet(controlLEDrow, trigPage + 1); // light the LED for the page we're on
          confirm = 255; // make sure confirm is turned off      
        }

        else // if we're on a trigger page
        {
          buttonMode = 0; // edit mode
          clearSelected();
          confirm = 255; // make sure confirm is turned off  
        }
        clearJust();    
      }

      else if (justreleased[39] && !pressed[32])
      {
        if (buttonMode < 5)
        {
          buttonMode = 5; // pot-shift mode
          clearSelected();
          ledsOff();
          controlLEDrow = B10000000;   
        }
        else
        { 
          buttonMode = 0;
          clearSelected();
        }
        clearJust();
      }

      //******EDIT MODE******
      // selecting the current button mode (edit mode only)
      else if(justpressed[33] && !pressed[32] && buttonMode < 4)
      {
        if (buttonMode == 0 && (seqRunning ||midiClock))
        {
          seqRunning = false;
          pulse = 0;
          seqTrueStep = 0;
          seqCurrentStep = 0;
          autoCounter = 0;
          if (midiNoteOut)
          {
            for (byte i = 0; i < 128; i++) // turn off all midi notes
              MIDI.sendNoteOff(i, 0, noteChannel);
          }
          noteOn = false;
          clearJust();
        }

        else if (buttonMode == 0 && !seqRunning)
        {
          //MIDI.sendSongPosition(0);
          seqRunning = true;
          noteOn = true;
          pulse = 0;
          seqTrueStep = 0;

          if (midiSyncOut)
          {
            MIDI.sendRealTime(Start); // send a midi clock start signal
            MIDI.sendRealTime(Clock); // send a midi clock pulse
          }
        }

        else
        {
          buttonMode = 0; // stepSelected
        }
        clearJust();        
      }

      else if(justpressed[34] && !pressed[32]  && buttonMode < 4)
      {
        clearSelected(); // first clear any selected steps
        buttonMode = 1; // stepMute
        clearJust();
      }
      else if(justpressed[35] && !pressed[32]  && buttonMode < 4)
      {
        clearSelected(); // first clear any selected steps
        buttonMode = 2; // stepTied
        clearJust();
      }
      else if(justpressed[36] && !pressed[32]  && buttonMode < 4)
      {
        clearSelected(); // first clear any selected steps
        buttonMode = 3; // stepSlide
        clearJust();
      }

      // shift-commands
      //******COMMON TO ALL******
      else if (pressed[32] && justpressed[37] && buttonMode != 4) // transpose pattern down by an octave (12 semitones) if we're not in trigger mode
      {
        if (buttonMode != 5 && buttonMode != 6) // we're not in pot-shift mode 
        {
          clearSelected();
          transpose(33, -12); // transpose function declared in HelperFuntions 
          // a value larger than 32 for the first variable means we're transposing the whole pattern, else pass the number of the step to be transposed
          // the second variable is the number of semitones to transpose by from +12 to -12 
        }
        else // if we are in pot-shift mode
        { 
          for (byte i = 0; i <32; i++)
          {
            if (checkSelected(i))
              transpose(i, -12);
          }
        }
        clearJust();
        shiftL = true;

      }

      else if (pressed[32] && justpressed[38] && buttonMode != 4) // transpose pattern up by an octave (12 semitones) if we're not in trigger mode
      {
        if (buttonMode < 5) // we're not in pot-shift mode
        {
          clearSelected();
          transpose(33, 12);
        }
        else // if we are in pot-shift mode
        {
          for (byte i = 0; i <32; i++)
          {
            if (checkSelected(i) && seqNotes[i] != B00000000)
              transpose(i, 12);
          } 
        }
        clearJust();
        shiftL = true;
      }

      // clear array currently being edited
      else if(pressed[32] && justreleased[33] && buttonMode != 4) 
      {
        clearEdit();
        shiftL = true;
        clearJust();
      }

      // clear EVERYTHING!!!
      else if(pressed[32] && justreleased[39])
      {
        if (buttonMode == 4)
        {
          shiftL = true;
          clearJust;
        }
        else if (!seqRunning) // restart the sequencer if it's not running
        {
          seqRunning = true;
          pulse = 0;
          seqTrueStep = 0;
          shiftL = true;
        }
        else
        {
          clearAll(); // defined in HelperFunctions
          followAction = 0; // don't add this to clearAll function sice clearAll gets called by the random pattern generator
          shiftL = true;
        }
      }

      // toggle automation (ie. off if it's on, and on if it's off)
      else if(pressed[39] && justreleased[38] && buttonMode < 5) // we're not in pot-shift mode
      {
        automate = !automate;    
        shiftR = true;
        clearJust();
      }

      else if(pressed[39] && justreleased[38] && buttonMode > 4) // we are in pot-shift mode
      {
        if (seqRunning && bpm < 220)
        {
          bpm++; // nudge the bpm faster
          bpmChange();
          showingNumbers = true;
          bpmPU = (pot6 != 550) ? 550 : 650; // reset the bpm
        }
        else // the sequencer is not running
        {
          pulse++;
        }    
        shiftR = true;
        clearJust();
      }

      else if(pressed[39] && justreleased[37] && buttonMode > 4) // we are in pot-shift mode
      {
        if (seqRunning && bpm > 45)
        {
          bpm--; // nudge the bpm slower
          bpmChange();
          showingNumbers = true;
          bpmPU = (pot6 != 550) ? 550 : 650;
        }
        else // the sequencer is not running
        {
          pulse = (pulse > 0) ? pulse - 1 : pulse; // subtract one pulse if it's value is more than 0;
        }    
        shiftR = true;
        clearJust();
      }

      else if (pressed[39] && justpressed[32])
      {
        clearJust();
        buttonMode = 7;
        seqRunning = false;
        noteOn = false;

        if (midiOn != 0 && midiNoteOut) // if we haven't turned off the midi note yet, do so now
        {
          MIDI.sendNoteOff(midiOn, 0, noteChannel); // turn off the note we last turned on
        }
      }

      // clear single notes && check for infinite random in pot-shift mode
      else if (pressed[32] && buttonMode != 4)
      {
        for (byte i = 0; i < 32; i++)
        {
          if (justpressed[i] && (seqNotes[i] != 0))
          {
            seqNotes[i] = B00000000;
            shiftL = true;
            if (i < 8)
            {
              seqStepMute[0] = bitClear(seqStepMute[0], i); //turn mute off 
              seqStepSelected[0] = bitClear(seqStepSelected[0], i); // turn selected off
            }
            else if (i < 16)
            {
              seqStepMute[1] = bitClear(seqStepMute[1], i%8); //turn mute off 
              seqStepSelected[1] = bitClear(seqStepSelected[1], i%8); // turn selected off
            }
            else if (i < 24)
            {
              seqStepMute[2] = bitClear(seqStepMute[2], i%8); //turn mute off 
              seqStepSelected[2] = bitClear(seqStepSelected[2], i%8); // turn selected off
            }
            else if (i < 32)
            {
              seqStepMute[3] = bitClear(seqStepMute[3], i%8); //turn mute off 
              seqStepSelected[3] = bitClear(seqStepSelected[3], i%8); // turn selected off         
            }  
          }
        }
        if (buttonMode > 4) // check for infinite random
        {
           updateNote();
          if (justpressed[33])
          {
            randomPattern(0); // defined in HelperFunctions - 0 = chromatic 
            followAction = 3; // create a new random chromatic pattern on step 0
            clearJust();
            shiftL = true;
          }
          else if (justpressed[34])
          {
            randomPattern(1); // defined in HelperFunctions - 1 = major scale
            followAction = 4; // create a new random major pattern on step 0
            clearJust();
            shiftL = true;
          }
          else if (justpressed[35])
          {
            randomPattern(2); // defined in HelperFunctions - 2 = minor scale
            followAction = 5; // create a new random minor pattern on step 0
            clearJust();
            shiftL = true;
          }
          else if (justpressed[36])
          {
            randomPattern(3); // defined in HelperFunctions - 3 = pentatonic
            followAction = 6; // create a new random pentatonic pattern on step 0
            clearJust();
            shiftL = true;
          }
        }      
      }



      else if (justpressed[37] && buttonMode != 4) // transpose pattern down by a semitone
      {
        if (buttonMode < 5) // we're not in pot-shift mode
        {
          clearSelected();
          transpose(33, -1);
        }
        else if (!pressed[39])// we are in pot-shift mode 
        {
          boolean mode6 = true;
          for (byte i = 0; i <32; i++)
          {
            if (checkSelected(i))
              transpose(i, -1); 
          }
          if (mode6)
            buttonMode = (buttonMode == 5) ? 6 : 5; // toggle between pot-shift mode (5) and pot-shift mode / accent (6)
          clearJust();
        }
      }

      else if (justpressed[38] && !pressed[39] && buttonMode != 4) // transpose pattern up by a semitone
      {
        if (buttonMode != 5 && buttonMode != 6) // we're not in pot-shift mode
        {
          clearSelected();
          transpose(33, 1);
        }
        else // we are in pot-shift mode 
        {
          boolean tempoTap = true;
          for (byte i = 0; i <32; i++)
          {
            if (checkSelected(i)) 
            {
              transpose(i, 1);
              tempoTap = false;
            }
          }
          if (tempoTap)
            tapTempo(); //defined HelperFunctions
        }
        clearJust();
      }



      // check if a step has been selected/deselected
      else if (buttonMode < 4)
      {
        buttonCheckSelected(); // defined in HelperFunctions     
      }

      // ******************************************
      //     Exlusive to:
      //     TRIGGER MODE inside the LOOP
      //     also inside the button-check structure
      // ******************************************

      else if (buttonMode == 4) // trigger mode
      { 


        for (byte i = 0; i < 32; i++)
        {
          if (justpressed[i])
          {
            i += (trigPage * 32); // there are 32 locations to a page
            if (i < 112) // there are only 112 save locations
            {
              if (pressed[32])
              {
                if (i != nowPlaying && checkToc(i))
                {       
                  clearMem = true;
                  confirm = i;
                  shiftL = true;
                  clearJust;
                }
                else
                {
                  shiftL = true;
                  clearJust;
                  confirm = 255;
                }  
              }
              else
              {                  
                if (i == confirm)
                {
                  if (clearMem)
                  {
                    clearMem = false;
                    tocClear(confirm);
                    confirm = 255; // turns confirm off - we chose 255 because it's a value that can't be arrived at with a buttonpress
                    clearJust;
                    longPress = 0; 
                  }
                  else
                  {
                    save = true; // we're go to save a preset
                    pageNum = confirm*4; // the number of the EEPROM page we want to write to is the number of the button times 4 - one memory location is 4 pages long
                    tocWrite(confirm); // write a new toc that adds the current save location
                    clearJust;
                    confirm = 255; // turns confirm off                     
                  }
                }
                else if (confirm == 255)
                {
                  longPress = millis();
                  save = false;
                  recall = false;
                  clearJust();
                }
                else
                {
                  confirm = 255;
                  save = false;
                  recall = false;
                  cancelSave = true;
                  clearJust();
                }
              }
            }
            clearJust();
          }

          else if (pressed[i])
          {
            i += (trigPage * 32); // multiply the trigpage by 32
            if (i < 112) // there are only 112 save locations
            {
              if (longPress != 0 && (millis() - longPress) > longPressDur) // has the button been pressed long enough
              {
                if(!checkToc(i))
                {
                  save = true; // we're go to save a preset
                  pageNum = i*4; // the number of the EEPROM page we want to write to is the number of the button times 4 - one memory location is 4 pages long
                  tocWrite(i); // write a new toc that adds the current save location
                  longPress = 0;
                }
                else // a patch is already saved in that location
                {
                  confirm = i;
                  longPress = 0;
                }
              }
            }
          }
          else if (justreleased[i]) // we don't have to check if save is true, since we wouldn't get here if it were
          {
            i += (trigPage * 32); // multiply the trigpage by 32 and add to to i
            if (i < 112) // there are only 112 save locations
            {
              if (cancelSave)
              {
                cancelSave = false;
              }
              else if (checkToc(i) && confirm == 255)
              {
                recall = true; // we're ready to recall a preset
                pageNum = i*4; // one memory location is 4 pages long
                clearJust();
                if (midiTrigger)
                {
                  MIDI.sendNoteOn(i, 127, triggerChannel); // send a pattern trigger note on triggerChannel (default is channel 10)
                }
                cued = i; // the pattern that will play next is i - need this to blink lights
                head = i; // the first pattern in a series of chained patterns
                followAction == 0; // allow the next loaded patch to prescribe to followAction
              }
            }
          }  
        } 
        if (pressed[32])
        {
          if (justpressed[33])
          {
            trigPage = 0;
            shiftL = true;    
          }  
          else if (justpressed[34])
          {
            trigPage = 1;
            shiftL = true;  
          }
          else if (justpressed[35])
          {
            trigPage = 2;
            shiftL = true;  
          }
          else if (justpressed[36])
          {
            trigPage = 3;
            shiftL = true;
          }
          else if (justpressed[37]) // set followAction to 2 - return to the head
          {
            if (followAction != 2)
            {
              followAction = 2;
              saved = false; // these follow actions only become active once the patch has been saved and loaded
              shiftL = true;
            }
            else
            {
              followAction = 0;
              shiftL = true;
            } 
          }
          else if (justpressed[38]) // set followAction to 1 - play the next pattern
          {
            if (followAction != 1)
            {
              followAction = 1;
              saved = false; // these follow actions only become active once the patch has been saved and loaded
              shiftL = true;
            }
            else
            {
              followAction = 0;
              shiftL = true;
            } 
          }
          controlLEDrow = B00000001; // light the LED for trigger mode
          controlLEDrow = bitSet(controlLEDrow, trigPage + 1); // light the LED for the page we're on
        }
        else if (justpressed[33]) // reverse playback
        {
          seqNextStep = (seqNextStep == 1) ? -1 : 1;
          clearJust();
        }

        else if (pressed[34]) // step repeat
        {
          seqNextStep = 0;
        }

        else if (justreleased[34])
        {
          seqNextStep = 1;
          clearJust();
        }

        else if (pressed[35]) // random step
        {
          seqNextStep = random(0, 32);
        }

        else if (justreleased[35])
        {
          seqNextStep = 1;
          clearJust();
        }

        else if (justpressed[38]) // pull things back in line
        {
          seqCurrentStep = seqTrueStep;
          clearJust();
        }

        if (save)
        {
          savePatch(); // defined in HelperFunctions - everything required to save a patch/pattern
        }  
      }
      // ******************************************
      //     end of TRIGGER MODE inside the LOOP
      // ******************************************

      // ******************************************
      //     Exlusive to:
      //     POT-SHIFT MODE inside the LOOP
      //      (Excluding shift commands)
      //     also inside the button-check structure
      // ******************************************
      else if (buttonMode > 4) //  we're in pot-shift mode
      {
        updateNote();

        buttonCheckSelected(); // defined in HelperFunctions
        if (justpressed[33])
        {
          randomPattern(0); // defined in HelperFunctions - 0 = chromatic 
          followAction = 0; // loop
          clearJust();
        }
        else if (justpressed[34])
        {
          randomPattern(1); // defined in HelperFunctions - 1 = major scale
          followAction = 0; // loop
          clearJust();
        }
        else if (justpressed[35])
        {
          randomPattern(2); // defined in HelperFunctions - 2 = minor scale
          followAction = 0; // loop
          clearJust();
        }
        else if (justpressed[36])
        {
          randomPattern(3); // defined in HelperFunctions - 3 = pentatonic
          followAction = 0; // loop
          clearJust();
        }
      }
      // *****************************************
      //     end of POT-SHIFT MODE inside the LOOP
      // *****************************************

    }

    // do this after all of our shift commands - we don't want to make a selection when the shift button is released

    else if (justreleased[32] && shiftL)
      shiftL = false;

    else if (justreleased[39] && shiftR)
    {
      shiftR = false;
      updateAuto = false;
      showingNumbers = false;
    }
    // ******************************************
    //     LED setup: work out what is lit
    //     end of button check if/else structure
    // ******************************************
    if (showingNumbers)
    {
      showNumber(bpm);
    }
    else
    {
      if (buttonMode != 4) // we're not in trigger mode
      {
        // we always start with active (not 0) steps and then work out what to blink or flash from there
        for (byte i = 0; i <32; i++)
        {
          if (seqNotes[i] != 0)
          {
            if (i < 8)
              LEDrow1 = bitSet(LEDrow1, i);
            else if (i < 16)
              LEDrow2 = bitSet(LEDrow2, i%8);
            else if (i < 24)
              LEDrow3 = bitSet(LEDrow3, i%8);
            else if (i < 32)
              LEDrow4 = bitSet(LEDrow4, i%8);
          }  
        }
      }

      if (buttonMode != 4) // blink selected steps if we're not in trigger mode
      {
        // blink step selected leds
        if (millis() > ledBlink + 100)
        {
          LEDrow1 ^= *edit[0];
          LEDrow2 ^= *edit[1];
          LEDrow3 ^= *edit[2];
          LEDrow4 ^= *edit[3];

          if (millis() > ledBlink + 200)
            ledBlink = millis(); 
        } 
      }

      if (buttonMode == 4) // we're in trigger mode
      {

        LEDrow1 = toc[trigPage*4];
        LEDrow2 = toc[(trigPage * 4) + 1];
        LEDrow3 = toc[(trigPage * 4) + 2];
        LEDrow4 = toc[(trigPage * 4) + 3];

        // slow blink nowPlaying
        if (nowPlaying < 255 && nowPlaying / 32 == trigPage) // we use 255 to turn nowPlaying "off", and only show if we're on the appropriate trigger page
        {
          if (millis() > ledBlink + 200)
          {
            if (nowPlaying % 32 < 8)
              LEDrow1 ^= 1<<(nowPlaying%8);
            else if (nowPlaying % 32 < 16)
              LEDrow2 ^= 1<<(nowPlaying%8);
            else if (nowPlaying % 32 < 24)
              LEDrow3 ^= 1<<(nowPlaying%8);
            else if (nowPlaying % 32 < 32)
              LEDrow4 ^= 1<<(nowPlaying%8);

            if (millis() > ledBlink + 400)
              ledBlink = millis(); 
          }
        } 

        // (fast) blink the pattern that's cued to play next
        if (cued < 255 && cued / 32 == trigPage) // we use 255 turn cued "off", and only show if we're on the appropriate trigger page
        {
          if (millis() > ledBlink2 + 100)
          {
            if (cued % 32 < 8)
              LEDrow1 ^= 1<<(cued%8);
            else if (cued % 32 < 16)
              LEDrow2 ^= 1<<(cued%8);
            else if (cued % 32 < 24)
              LEDrow3 ^= 1<<(cued%8);
            else if (cued % 32 < 32)
              LEDrow4 ^= 1<<(cued%8);  

            if (millis() > ledBlink2 + 200)
              ledBlink2 = millis(); 
          }
        }

        // (fast) blink the location that needs to be confirmed to overwrite
        else if (confirm < 255 && confirm / 32 == trigPage)
        {
          if (millis() > ledBlink2 + 100)
          {
            if (confirm % 32 < 8)
              LEDrow1 ^= 1<<(confirm%8);
            else if (confirm % 32 < 16)
              LEDrow2 ^= 1<<(confirm%8);
            else if (confirm % 32 < 24)
              LEDrow3 ^= 1<<(confirm%8);
            else if (confirm % 32 < 32)
              LEDrow4 ^= 1<<(confirm%8);

            if (millis() > ledBlink2 + 200)
              ledBlink2 = millis(); 
          }
        }

        bitClear(controlLEDrow, 6);
        bitClear(controlLEDrow, 5);
        if (followAction == 1 || followAction == 2)  // light the LED for followAction 1 or 2
            controlLEDrow = bitSet(controlLEDrow, 7 - followAction);
      }

      // this blinks the led for the current step
      stepLEDrow1 = (seqCurrentStep < 8) ? B00000001 << seqCurrentStep : B00000000;
      stepLEDrow2 = (seqCurrentStep > 7 && seqCurrentStep < 16) ? B00000001 << (seqCurrentStep % 8) : B00000000;
      stepLEDrow3 = (seqCurrentStep > 15 && seqCurrentStep < 24) ? B00000001 << (seqCurrentStep % 8) : B00000000;
      stepLEDrow4 = (seqCurrentStep > 23 && seqCurrentStep < 32) ? B00000001 << (seqCurrentStep % 8) : B00000000;    
      LEDrow1 ^= stepLEDrow1;
      LEDrow2 ^= stepLEDrow2;
      LEDrow3 ^= stepLEDrow3;
      LEDrow4 ^= stepLEDrow4;  



      if (buttonMode > 4) // if we're in pot-shift mode, turn on the LEDs for the infinite random options
      {
        if (followAction > 2)
          switch (followAction)
          {
          case 3: // chromatic
            controlLEDrow = bitSet(controlLEDrow, 1);
            break;
          case 4: // major
            controlLEDrow = bitSet(controlLEDrow, 2);
            break;
          case 5: // minor
            controlLEDrow = bitSet(controlLEDrow, 3);
            break;
          case 6: // pentatonic
            controlLEDrow = bitSet(controlLEDrow, 4);
            break;
          }
      }

      LEDrow5 = controlLEDrow; // light the LED for the mode we're in
    }
    // this function shifts out the the 5 bytes corresponding to the led rows
    // declared in ShiftOut (see above)
    updateLeds();

    if (buttonMode < 4 && lastButtonMode < 5) // we're not in trigger mode or pot-shift mode
    {
      //update the selected steps with the value of pot6
      for (byte i = 0; i < 32; i++)
      {
        if (checkSelected(i))
          seqNotes[i] = tempNote;
      }
    }

    clearJust();


    // *************************************
    //     SEQUENCER inside the LOOP
    // *************************************
    if (seqRunning)
    {
      if (currentTime == 0) // the fist time through the loop
      {
        stepGo = true;
        //MIDI.sendSongPosition(0);
        if (midiSyncOut)
        {
          MIDI.sendRealTime(Start); // send a midi clock start signal
          MIDI.sendRealTime(Clock); // send a midi clock pulse
        }
        currentTime = millis();
        sixteenthStartTime = currentTime;
        pulseStartTime = currentTime;
        swing16thDur = map(swing, 0, 255, sixteenthDur, ((2*sixteenthDur)/3)*2);
      }
      else  
      {
        currentTime = millis();

        if (clockPulse < 11) // if the clockPulse is 11, wait for the next 8th (16th may swing) so they stay nicely in sync
        {
          if (currentTime - pulseStartTime >= clockPulseDur) // is it time to fire a clock pulse?
          {
            pulseStartTime = currentTime;
            if (midiSyncOut)
            {
              MIDI.sendRealTime(Clock); // send a midi clock pulse
            }
            clockPulse++; // advance the pulse counter
            autoCounter = (autoCounter < 191) ? autoCounter + 1 : 0;          
          }  
        }

        if (currentTime - sixteenthStartTime >= swing16thDur)
        { 
          //swing16thDur = (swing16thDur > sixteenthDur) ? ((2*sixteenthDur) - swing16thDur) : map(swing, 0, 255, sixteenthDur, ((2*sixteenthDur)/3)*2);
          //onDur = (onDur > swing16thDur) ? swing16thDur : onDur;
          stepGo = true;
          sixteenthStartTime = currentTime;
          seqTrueStep = (seqTrueStep+1)%32;
          if (seqTrueStep%2 != 0) // it's an uneven step, ie. the 2nd shorter 16th of the swing-pair
            swing16thDur = (2*sixteenthDur) - swing16thDur;
          else // it's an even step ie. the first longer 16th of the swing-pair
          {
            swing16thDur = map(swing, 0, 255, sixteenthDur, ((2*sixteenthDur)/3)*2);
            //MIDI.sendSongPosition(seqCurrentStep); 

            while (clockPulse < 11)
            {
              if (midiSyncOut)
              {
                MIDI.sendRealTime(Clock); // send a midi clock pulse
              }
              clockPulse++;
              delay(1);
            }

            if (clockPulse = 11)
            {
              pulseStartTime = currentTime;
              clockPulse = 0;
              if (midiSyncOut)
              {
                MIDI.sendRealTime(Clock); // send a midi clock pulse
              }  
            }
          }
          onDur = (onDur > swing16thDur) ? swing16thDur : onDur;
          seqCurrentStep = (seqCurrentStep + seqNextStep)%seqLength; // advance the step counter
          if (seqCurrentStep == 0)
          {
            autoCounter = 0;
          }
          else  
            autoCounter = (autoCounter < 191) ? autoCounter + 1 : 0;
        }
      }
    }
    // is it time to move to the next step?
    if ((stepGo || seqMidiStep) && on)
    {
      if (seqCurrentStep == 0 || seqCurrentStep == 16)
      {
        if (recall)
        {
          loadPatch(cued); // defined in HelperFunctions - everything that happens with a patch/pattern change
        }
        //deal with the follow actions if we're on the first step
        else if (seqCurrentStep == 0 && followAction != 0)
        {
          switch (followAction)
          {
          case 1: // play the next pattern      
            if (checkToc(nowPlaying + 1) && saved == true) // check if there is a pattern stored in the next location
            {
              loadPatch(nowPlaying + 1); // load the next patch
            }
            break;
          case 2:
            if (saved == true) // we don't want to go over page breaks (30 is the second to last location on the page), and check if there is a pattern stored in the next location
            {
              loadPatch(head); // load the patch marked as the head
            }
            break;
          case 3:
            randomPattern(0); // chromatic
            break;
          case 4:
            randomPattern(1); // major
            break;
          case 5:
            randomPattern(2); // minor
            break;
          case 6:
            randomPattern(3); // pentatonic
            break;
          }
        }
      }
      stepGo = false;
      offTime = sixteenthStartTime + onDur; // schedule when the note should be turned off
      if (seqNotes[seqCurrentStep] != 0)
        noteOn = true;
      seqMidiStep = false;
      on = false; // toggle, because we only want this section of code to run once    
      slide = false; // slide has to be turned off when the next note starts
      if (!checkTied(seqCurrentStep)) // do this code only if this step is not tied (to the previous)
      {
        if (seqNotes[seqCurrentStep] == 0 || checkMute(seqCurrentStep))
        {
          noteOn = false;
        }
        //else if (!checkMute(seqCurrentStep)) // if the step is not active
        //  noteOn = false;        
        else // if we got this far, we can safely send out a midi note 
        {
          syncPhaseInc = pgm_read_word_near(midiTable +(seqNotes[seqCurrentStep]));
          if (midiOn != 0 && midiNoteOut) // if we haven't turned off the midi note yet, do so now
          {
            MIDI.sendNoteOff(midiOn, 0, noteChannel); // turn off the note we last turned on
          }

          midiOn = seqNotes[seqCurrentStep];
          if (midiNoteOut)
          { 
            if (checkAccent(seqCurrentStep)) // does the step have an accent?
              MIDI.sendNoteOn(midiOn, 127, noteChannel); // send a note with velocity 127
            else
              MIDI.sendNoteOn(midiOn, 100, noteChannel); // send a note with velocity 100
          }
        }
        //osc1.incr = getMidiNotePhaseIncrement(seqNotes[seqCurrentStep]);
      }
      else
      {
        // osc1.incr = getMidiNotePhaseIncrement(seqTiedNote); // if it is tied, get the look-up value for the note to play from seqTiedNote
      } 
      if (checkTied((seqCurrentStep + seqNextStep)%seqLength) && !checkTied(seqCurrentStep) && !checkMute(seqCurrentStep)) // check if the next step is tied, but this one not, and this step is not muted
        seqTiedNote = seqNotes[seqCurrentStep]; // if so, assign this note's lookup-value to seqTiedNote

    }
    if (millis() >= offTime)
    {
      if (!on)
      {
        if (!checkSlide(seqCurrentStep)) // check that the step isn't marked to slide
        {
          on = true; // toggle, because we only want this section of code to run once 
          if (!checkTied((seqCurrentStep + seqNextStep)%seqLength)) // turn off the note unless the next step is tied (to this one)
          {
            noteOn = false;
            if (midiOn != 0 && midiNoteOut)
            {
              MIDI.sendNoteOff(midiOn, 0, noteChannel); // turn off the note we last turned on
              midiOn = 0;
            }
          }
        }
        else if (!slide)// this step is marked as a slide, but we haven't toggled slide yet (we want to run the following code once only per slide)
        {
          slide = true;
          on = true; // toggle, because we only want this section of code to run once      
          pitchTarget = pgm_read_word_near(midiTable + (seqNotes[(seqCurrentStep + seqNextStep)%seqLength]));
        }
      }
      //do the following each time though the slide loop
      else if (slide)
      {
        if (pitchTarget > syncPhaseInc) 
          syncPhaseInc += (pitchTarget - syncPhaseInc)/((sixteenthStartTime + swing16thDur) - millis()); // work out by how much to change the pitch to get to the target pitch in time 

        else // the target must be smaller than the current increment
        {
          syncPhaseInc -= (syncPhaseInc - pitchTarget)/((sixteenthStartTime + swing16thDur) - millis());
        }
      }  
    }
  }
}

















































































































































